import { createStore, applyMiddleware, compose } from 'redux';
import { composeWithDevTools } from '../../../npm-package/logOnly'; // 'redux-devtools-extension';
import thunk from 'redux-thunk';
import invariant from 'redux-immutable-state-invariant';
import reducer from '../reducers';
import * as actionCreators from '../actions/counter';

import SyntheticEvent from 'react/lib/SyntheticEvent';


import Immutable from 'immutable';

export const Record = new Immutable.Record({a:1, b:2});

function mark(data, type, isArray) {
  return {
    data: isArray ? data.toArray() : data.toObject(),
    __remotedevType__: type
  };
}

function extract(data, type) {
  return {
    data: Object.assign({}, data),
    __remotedevType__: type
  };
}

function replacer(key, value) {
  console.log(key, value)
  if (value instanceof Immutable.Record) return mark(value, 'ImmutableRecord');
  if (value instanceof Immutable.Range) return extract(value, 'ImmutableRange');
  if (value instanceof Immutable.Repeat) return extract(value, 'ImmutableRepeat');
  if (Immutable.OrderedMap.isOrderedMap(value)) return mark(value, 'ImmutableOrderedMap');
  if (Immutable.Map.isMap(value)) return mark(value, 'ImmutableMap');
  if (Immutable.List.isList(value)) return mark(value, 'ImmutableList', true);
  if (Immutable.OrderedSet.isOrderedSet(value)) return mark(value, 'ImmutableOrderedSet', true);
  if (Immutable.Set.isSet(value)) return mark(value, 'ImmutableSet', true);
  if (Immutable.Seq.isSeq(value)) return mark(value, 'ImmutableSeq', true);
  if (Immutable.Stack.isStack(value)) return mark(value, 'ImmutableStack', true);
  return value;
}

function reviver(key, value) {
  if (typeof value === 'object' && value !== null && '__remotedevType__'  in value) {
    switch (value.__remotedevType__) {
      case 'ImmutableRecord': return Immutable.Record(value.data);
      // ...
      default: return Immutable.fromJS(value.data);
    }
  }
  return value;
}

function testMiddleware() {
  return store => next => action => {
    console.log(action);
    return next(action);
  }
}

export let isMonitorAction;
export default function configureStore(preloadedState) {
  function serialize(key, value) {
    console.log(value);
    if (typeof value === 'object') {
      return {...value, __somePrivate: true};
    }
    return value;
  }

  function deserialize(obj) {
    console.warn('Deserializing %o', obj);
    delete obj.__somePrivate;
    return obj;
  }

  console.log(actionCreators)
  const composeEnhancers = composeWithDevTools({ latency: 5000, maxAge: 5000, actionCreators, serialize: {
    // options: true,
    // replacer
    /*replacer: (key, value) => {
      if (value && value instanceof SyntheticEvent) return {};
      return value;
    }*/
    immutable: Immutable, refs: [Record]
  }, deserializeState: deserialize });
  const store = createStore(reducer, preloadedState, composeEnhancers(
    applyMiddleware(thunk, testMiddleware())
  ));

  if (module.hot) {
    // Enable Webpack hot module replacement for reducers
    module.hot.accept('../reducers', () => {
      const nextReducer = require('../reducers');
      store.replaceReducer(nextReducer);
    });
  }
/*
  
  let i = true;
  store.subscribe(() => {
    if (i) {
      i = false;
      setTimeout(() => store.dispatch({ type: 'INCREMENT_COUNTER' }), 600);
      setTimeout(() => store.dispatch({ type: 'INCREMENT_COUNTER' }), 0);
    }
  })
*/
  window.store = store;
  return store;
}
