import React, { Component, PropTypes } from 'react';

const withDevTools = (
  // process.env.NODE_ENV === 'development' &&
  typeof window !== 'undefined' && window.devToolsExtension
);

class Counter extends Component {
  constructor() {
    super();
    this.state = { counter: 0 };
    
    this.increment = this.increment.bind(this);
    this.decrement = this.decrement.bind(this);
  }

  componentWillMount() {
    setTimeout(() => {
      this.devTools = window.devToolsExtension.connect();
      this.unsubscribe = this.devTools.subscribe((message) => {
        console.log('message',message)
        // Implement monitors actions.
        // For example time traveling:
        if (message.type === 'DISPATCH' && message.payload.type === 'JUMP_TO_STATE') {
          this.setState(JSON.parse(message.state));
        }
      });
    }, 1000)
  }

  componentWillUnmount() {
    if (withDevTools) {
      this.unsubscribe(); // Use if you have other subscribers from other components.
      window.devToolsExtension.disconnect(); // If there aren't other subscribers.
    }
  }

  increment() {
    const state = { counter: this.state.counter + 1 };
    this.devTools.send('increment', state);
    this.setState(state);
  }

  decrement() {
    const state = { counter: this.state.counter - 1 };
    this.devTools.send('decrement', state);
    this.setState(state);
  }

  render() {
    const { counter } = this.state;
    return (
      <p>
        Clicked: {counter} times
        {' '}
        <button onClick={this.increment}>+</button>
        {' '}
        <button onClick={this.decrement}>-</button>
      </p>
    );
  }
}

export default Counter;
